package com.carlosbcruz.internet;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.net.URL;
import java.net.URLConnection;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class InternetFile extends JFrame {

	private static final long serialVersionUID = 1L;

	JLabel urlLabel;
	JTextField inputText;
	JLabel outputFile;
	JTextField outputText;
	JButton downloadButton;
	JLabel status;
	JPanel firstPane;
	private String globalUrlString;
	private String globalProxyServer;
	private int globalProxyPort;
	static int BLOCK_SIZE = 4096;

	/**
	 * Default constructor.
	 */
	public InternetFile() {

		this("", "", 0);
	}

	/**
	 * Constructor.
	 * 
	 * @param urlString
	 *            The URL.
	 */
	public InternetFile(String urlString) {

		this(urlString, "", 0);
	}

	/**
	 * Construct the Internet File program.
	 * 
	 * @param urlString
	 *            The URL.
	 * @param proxyServer
	 *            The proxy server.
	 * @param proxyPort
	 *            The proxy port.
	 */
	public InternetFile(String urlString, String proxyServer, int proxyPort) {

		urlLabel = new JLabel("Url:");
		inputText = new JTextField("", 50);
		outputFile = new JLabel("Output:");
		outputText = new JTextField("", 10);
		downloadButton = new JButton("Download");
		status = new JLabel("Status:");
		firstPane = new JPanel();
		globalUrlString = urlString;
		globalProxyServer = proxyServer;
		globalProxyPort = proxyPort;
		getContentPane().setLayout(new BorderLayout());
		firstPane.add(urlLabel);
		firstPane.add(inputText);
		firstPane.add(outputFile);
		firstPane.add(outputText);
		firstPane.add(downloadButton);
		getContentPane().add(firstPane, "Center");
		getContentPane().add(status, "South");
		downloadButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {

				String inputUrl = inputText.getText();
				String outputFileName = outputText.getText();
				System.out.println(inputUrl);
				System.out.println(outputText);
				GetURLFile(inputUrl, outputFileName, "", 80);
			}

		});
		addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e) {

				System.exit(0);
			}

		});
		pack();
		setVisible(true);
	}

	/**
	 * Set the URL.
	 * 
	 * @param urlString
	 *            The URL.
	 */
	public void SetURL(String urlString) {

		globalUrlString = urlString;
	}

	/**
	 * Get URL.
	 * 
	 * @return The URL.
	 */
	public String GetURL() {

		return globalUrlString;
	}

	/**
	 * Set the port an server.
	 * 
	 * @param proxyServer
	 *            The server.
	 * @param proxyPort
	 *            The port.
	 */
	public void SetProxy(String proxyServer, int proxyPort) {

		globalProxyServer = proxyServer;
		globalProxyPort = proxyPort;
	}

	/**
	 * Get the server.
	 * 
	 * @return The server.
	 */
	public String GetProxyServer() {

		return globalProxyServer;
	}

	/**
	 * Get port.
	 * 
	 * @return The port.
	 */
	public int GetProxyPort() {

		return globalProxyPort;
	}

	/**
	 * Get a file.
	 * 
	 * @return True if it was ok.
	 */
	public boolean GetURLFile() {

		return GetURLFile(globalUrlString, "", globalProxyServer,
				globalProxyPort);
	}

	/**
	 * Get a file.
	 * 
	 * @param urlString
	 *            The URL.
	 * @return True if it was ok.
	 */
	public boolean GetURLFile(String urlString) {

		return GetURLFile(urlString, "", globalProxyServer, globalProxyPort);
	}

	/**
	 * Get a file.
	 * 
	 * @param urlString
	 *            The URL.
	 * @param localFileName
	 *            The file name.
	 * @return True if it was ok.
	 */
	public boolean GetURLFile(String urlString, String localFileName) {

		return GetURLFile(urlString, localFileName, globalProxyServer,
				globalProxyPort);
	}

	/**
	 * Retrieve the file.
	 * 
	 * @param urlString
	 *            The URL.
	 * @param localFileName
	 *            The file name.
	 * @param proxyServer
	 *            The proxy.
	 * @param proxyPort
	 *            The port.
	 * @return True if it was ok.
	 */
	public boolean GetURLFile(String urlString, String localFileName,
			String proxyServer, int proxyPort) {

		BufferedInputStream fileStream;
		RandomAccessFile outFile;
		fileStream = null;
		outFile = null;

		try {

			URL theUrl;

			if (proxyServer.length() > 0 && proxyPort > 0)
				theUrl = new URL("http", proxyServer, proxyPort, urlString);
			else
				theUrl = new URL(urlString);

			if (localFileName.length() == 0)
				localFileName = getFileName(urlString);

			status.setText("Attempting to connect to " + theUrl);

			pack();

			URLConnection con = theUrl.openConnection();

			fileStream = new BufferedInputStream(con.getInputStream());
			outFile = new RandomAccessFile(localFileName, "rw");
			status.setText("Downloading to local file " + localFileName);

			pack();

			byte bytesIn[] = new byte[BLOCK_SIZE];

			int howManyBytes;

			while ((howManyBytes = fileStream.read(bytesIn)) >= 0) {

				outFile.write(bytesIn, 0, howManyBytes);

				status.setText("Bytes: " + bytesIn + " - " + howManyBytes);

				pack();
			}

			fileStream.close();
			outFile.close();

			status.setText("Finished downloading file to " + localFileName);

			pack();

		} catch (IOException ioException) {

			status.setText("IOException: " + ioException);
			pack();
			return false;

		} finally {

			try {

				fileStream.close();
				outFile.close();

			} catch (Exception exception) {

				status.setText("Exception: " + exception);
				pack();
				return false;

			}
		}
		return true;
	}

	/**
	 * Get the file name of an URL.
	 * 
	 * @param urlString
	 *            The URL.
	 * @return The name.
	 */
	private static String getFileName(String urlString) {

		int lastSlash = urlString.lastIndexOf('/');
		String tempString;
		if (lastSlash >= 0)
			tempString = urlString.substring(lastSlash + 1);
		else
			tempString = new String("");
		if (tempString.length() == 0)
			tempString = new String("Default.txt");
		return tempString;
	}

	/**
	 * Main method.
	 * 
	 * @param args
	 *            Arguments.
	 */
	public static void main(String args[]) {

		new InternetFile();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.Component#toString()
	 */
	public String toString() {

		if (globalProxyServer.length() == 0)
			return globalUrlString;
		else
			return globalUrlString + " (proxy: " + globalProxyServer
					+ " at port " + globalProxyPort;
	}
}
