/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.pageschanges;

import java.awt.FlowLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JTextArea;

/**
 * Verify pange changes.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class VerifyPageChanges {

	/**
	 * Connect to a specific url
	 * 
	 * @param link
	 *            the url
	 */
	public void verifyUrl(List links) {

		// Write text task bar.
		for (int i = 0; i < links.size(); i++) {
			System.out.print("-");
		}
		System.out.println();

		Iterator i = links.iterator();

		while (i.hasNext()) {

			LinkBean link = (LinkBean) i.next();

			System.out.print("#");

			String url = link.getUrl();

			URL urlInstance = null;
			try {
				urlInstance = new URL(url);
			} catch (MalformedURLException e) {
				link.setMessage("Error instantianting url: " + url + " - "
						+ e.getMessage());
				continue;
			}

			URLConnection connection = null;

			try {
				connection = urlInstance.openConnection();
			} catch (IOException e1) {

				if (!"Disconsider".equals(link.getException())) {

					if (link.getException().equals(e1.getMessage())) {
						continue;
					}
				}

				link.setMessage("Error opening a connection to url: " + url
						+ " - " + e1.getMessage());
				continue;
			}

			String header = connection.getHeaderField(0);

			if (!"Disconsider".equals(link.getHeader())) {

				if (header == null) {
					link.setMessage(url + " - Header returned is null");
					continue;
				}

				String httpHeader = new String(header.getBytes());
				if (!link.getHeader().equals(httpHeader)) {
					link.setMessage("Header dos not match: " + link.getHeader()
							+ " and " + httpHeader);
					continue;
				}

				continue;
			}

			InputStream input = null;
			try {
				input = connection.getInputStream();
			} catch (IOException e2) {

				if (!"Disconsider".equals(link.getException())) {

					if (link.getException().equals(e2.getMessage())) {
						continue;
					}
				}

				link.setMessage("Error opening the input stream: " + url
						+ " - " + e2.getMessage());
				continue;
			}

			BufferedInputStream buffered = new BufferedInputStream(input);

			long numberOfChars = 0;
			try {

				byte[] buffer = new byte[5000];

				int read = 0;
				while ((read = buffered.read(buffer)) != -1) {
					numberOfChars += read;
				}
			} catch (IOException e2) {

				if (!"Disconsider".equals(link.getException())) {

					if (link.getException().equals(e2.getMessage())) {
						continue;
					}
				}

				link.setMessage("Error reading stream: " + url + " - "
						+ e2.getMessage());
				continue;
			}

			if (numberOfChars != link.getPreviousContent()) {
				link.setMessage(url + " - Number of bytes does not match: "
						+ numberOfChars + " - " + link.getPreviousContent());
				continue;
			}
		}

		String output = new String();

		Iterator j = links.iterator();
		int counter = 0;
		while (j.hasNext()) {
			LinkBean link = (LinkBean) j.next();

			if (link.getMessage() != null) {
				output += link.getMessage() + "\n";
			} else {
				counter++;
			}
		}

		JFrame a = new JFrame();

		JTextArea t = new JTextArea("Successful links: " + counter + "\n"
				+ output);
		a.getContentPane().setLayout(new FlowLayout());
		a.getContentPane().add(t);
		a.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e) {

				System.exit(0);
			}

		});
		a.pack();
		a.setLocation(100, 100);
		a.setVisible(true);
	}

	/**
	 * Main method.
	 * 
	 * @param args
	 *            no arguments considered.
	 */
	public static void main(String args[]) {

		VerifyPageChanges verify = new VerifyPageChanges();

		ArrayList links = new ArrayList();

		String link = null, header = null, exception = null, value = null;

		int counter = 1;
		do {

			link = PropertiesController.getStringProperty("link" + counter);
			header = PropertiesController.getStringProperty("header" + counter);
			exception = PropertiesController.getStringProperty("exception"
					+ counter);
			value = PropertiesController.getStringProperty("value" + counter);

			if (link != null && header != null && exception != null
					&& value != null) {
				links.add(new LinkBean(link, header, exception, Long
						.parseLong(value)));
			}
			counter++;
		} while (link != null && header != null && exception != null
				&& value != null);

		verify.verifyUrl(links);
	}
}
