/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.percurse.beans;

import java.util.Calendar;

/**
 * Fuel Bean Decorator
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class FuelViewBean {

	private FuelBean fuelBean;

	/**
	 * Decorate a specific bean
	 * 
	 * @param fuelBean
	 */
	public FuelViewBean(FuelBean fuelBean) {
		this.fuelBean = fuelBean;
	}

	/**
	 * Create a correspondent calendar to the bean date
	 * 
	 * @return the correspondent calendar
	 */
	public Calendar getCalendar() {

		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.DAY_OF_MONTH, getDay());
		calendar.set(Calendar.MONTH, getMonth());
		calendar.set(Calendar.YEAR, getYear());
		calendar.set(Calendar.HOUR_OF_DAY, 10);
		calendar.set(Calendar.MINUTE, 0);
		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MILLISECOND, 0);

		return calendar;
	}

	/**
	 * Get the day
	 * 
	 * @return Returns the day.
	 */
	public int getDay() {
		return fuelBean.getDay();
	}

	/**
	 * Get the month
	 * 
	 * @return Returns the month.
	 */
	public int getMonth() {
		return fuelBean.getMonth();
	}

	/**
	 * Get the year
	 * 
	 * @return Returns the year.
	 */
	public int getYear() {
		return fuelBean.getYear();
	}

	/**
	 * Retrieve the fuel cost
	 * 
	 * @return Returns the cost.
	 */
	public float getCost() {
		return fuelBean.getCost();
	}

	/**
	 * Get bean property
	 * 
	 * @return odometer
	 */
	public long getOdometer() {
		return fuelBean.getOdometer();
	}

	/**
	 * Get bean property
	 * 
	 * @return volume
	 */
	public float getVolume() {
		return fuelBean.getVolume();
	}

}
