/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.percurse.beans;

import java.util.Calendar;
import java.util.Date;

/**
 * Percurse Bean Decorator
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class PercurseViewBean {

	private PercurseBean percurseBean;

	private float partialCost;

	private long partialOdometer;

	/**
	 * Decorate a specific bean
	 * 
	 * @param percurseBean
	 */
	public PercurseViewBean(PercurseBean percurseBean) {
		this.percurseBean = percurseBean;
	}

	/**
	 * Create a correspondent calendar to the com.carlosbcruz.percurse start
	 * 
	 * @return the correspondent calendar
	 */
	public Calendar getStart() {

		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.DAY_OF_MONTH, getDay());
		calendar.set(Calendar.MONTH, getMonth());
		calendar.set(Calendar.YEAR, getYear());
		calendar.set(Calendar.HOUR_OF_DAY, getStartHour());
		calendar.set(Calendar.MINUTE, getStartMinute());
		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MILLISECOND, 0);

		return calendar;
	}

	/**
	 * Create a correspondent calendar to the com.carlosbcruz.percurse end
	 * 
	 * @return the correspondent calendar
	 */
	public Calendar getEnd() {

		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.DAY_OF_MONTH, getDay());
		calendar.set(Calendar.MONTH, getMonth());
		calendar.set(Calendar.YEAR, getYear());
		calendar.set(Calendar.HOUR_OF_DAY, getEndHour());
		calendar.set(Calendar.MINUTE, getEndMinute());
		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MILLISECOND, 0);

		return calendar;
	}

	/**
	 */
	public long getDuration() {

		Date start = getStart().getTime();
		Date end = getEnd().getTime();

		long startDate = start.getTime();
		long endDate = end.getTime();

		return (endDate - startDate) / (60L * 1000L);

	}

	/**
	 * Get internal atribute
	 * 
	 * @return Returns the partialCost.
	 */
	public float getPartialCost() {
		return partialCost;
	}

	/**
	 * Set internal atribute
	 * 
	 * @param partialCost
	 *            The partialCost to set.
	 */
	public void setPartialCost(float partialCost) {
		this.partialCost = partialCost;
	}

	/**
	 * Get internal atribute
	 * 
	 * @return Returns the partialOdometer.
	 */
	public long getPartialOdometer() {
		return partialOdometer;
	}

	/**
	 * Set internal atribute
	 * 
	 * @param partialOdometer
	 *            The partialOdometer to set.
	 */
	public void setPartialOdometer(long partialOdometer) {
		this.partialOdometer = partialOdometer;
	}

	/**
	 * Get bean property
	 * 
	 * @return category
	 */
	public String getCategory() {
		return percurseBean.getCategory();
	}

	/**
	 * Get bean property
	 * 
	 * @return day
	 */
	public int getDay() {
		return percurseBean.getDay();
	}

	/**
	 * Get bean property
	 * 
	 * @return month
	 */
	public int getMonth() {
		return percurseBean.getMonth();
	}

	/**
	 * Get bean property
	 * 
	 * @return odometer
	 */
	public long getOdometer() {
		return percurseBean.getOdometer();
	}

	/**
	 * Get bean property
	 * 
	 * @return year
	 */
	public int getYear() {
		return percurseBean.getYear();
	}

	/**
	 * Get bean property
	 * 
	 * @return End Hour
	 */
	public int getEndHour() {
		return percurseBean.getEndHour();
	}

	/**
	 * Get bean property
	 * 
	 * @return End Minute
	 */
	public int getEndMinute() {
		return percurseBean.getEndMinute();
	}

	/**
	 * Get bean property
	 * 
	 * @return Start Hour
	 */
	public int getStartHour() {
		return percurseBean.getStartHour();
	}

	/**
	 * Get bean property
	 * 
	 * @return Start Minute
	 */
	public int getStartMinute() {
		return percurseBean.getStartMinute();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {

		StringBuffer buffer = new StringBuffer();

		buffer.append("PercurseViewBean(" + partialOdometer + " " + partialCost
				+ " " + getDuration() + " " + percurseBean + ")");

		return buffer.toString();
	}
}
