/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.replacestring;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

/**
 * Read the documents in memory
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class DocumentsReader {

	// The number of bytes to read
	private long lengthOfTask;

	// Actual file that is being read
	private long current;

	// Message of the current status.
	private String statMessage;

	// List of files to open
	private ArrayList fileNames;

	// To read the files
	private File file;
	private FileReader fileReader;
	private BufferedReader bufferedReader;

	Vector documentListeners = new Vector();

	boolean stop = false;

	/**
	 * Constructor. Get the file list and count the total number of bytes.
	 */
	DocumentsReader(ArrayList fileNames) {

		// Get the file names
		this.fileNames = fileNames;

		// Count the number of bytes to read
		long counter = 0;
		Iterator files = fileNames.iterator();
		while (files.hasNext()) {
			file = new File((String) files.next());
			counter += file.length();
		}
		lengthOfTask = counter;
	}

	/**
	 * Add Listeners
	 */
	void addDocumentoCreationListener(DocumentCreationListener listener) {
		documentListeners.add(listener);
	}

	/**
	 * Remove Listeners
	 */
	void removeDocumentoCreationListener(DocumentCreationListener listener) {
		documentListeners.remove(listener);
	}

	/**
	 * Notify Listeners
	 */
	void notifyCreation(InternalDocument document) {
		Enumeration listenersEnumeration = documentListeners.elements();
		while (listenersEnumeration.hasMoreElements()) {
			DocumentCreationListener listener = (DocumentCreationListener) listenersEnumeration
					.nextElement();
			listener.documentReaded(document);
		}
	}

	/**
	 * Start the process of reading the files through an inner class ActualTask.
	 */
	void go() {
		current = 0;
		final SwingWorker worker = new SwingWorker() {
			public Object construct() {
				return new ActualTask();
			}
		};
		worker.start();
	}

	/**
	 * Get the total number of bytes to read
	 */
	long getLengthOfTask() {
		return lengthOfTask;
	}

	/**
	 * Get the total number of bytes readed until the moment of this method
	 * call.
	 */
	long getCurrent() {
		return current;
	}

	/**
	 * Stop the reading
	 */
	void stop() {
		current = lengthOfTask;
		stop = true;
	}

	/**
	 * Inform if the read is completed
	 */
	boolean done() {
		return (current >= lengthOfTask);
	}

	/**
	 * Return the current message status
	 */
	String getMessage() {
		return statMessage;
	}

	/**
	 * Class to perform the reading
	 */
	class ActualTask {

		/**
		 * Constructor to perform the reading
		 */
		ActualTask() {

			stop = false;

			Iterator files = fileNames.iterator();
			while (files.hasNext()) {

				// Read the file
				String fileName = (String) files.next();
				file = new File(fileName);

				boolean errorFound = false;
				try {

					fileReader = new FileReader(file);
					bufferedReader = new BufferedReader(fileReader);

					// Content of the file
					StringBuffer content = new StringBuffer();
					// Current line
					String line = new String();
					while ((line = bufferedReader.readLine()) != null) {
						content.append(line);
						content.append("\r\n");
						// update the number of bytes read
						current += line.length() + 1;
						statMessage = "Completed " + current + " of "
								+ lengthOfTask;
					}

					notifyCreation(new InternalDocument(fileName, content, file
							.length()));

				} catch (Exception e) {
					errorFound = true;
					e.printStackTrace();
				}

				// Stop if it is pressed Cancel
				if (stop)
					return;

			}
			current = lengthOfTask;
			statMessage = "Completed " + current;

		}
	}

}
