/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.replacestring;

/**
 * Listem to the "Save Files" menu item action
 */

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.ProgressMonitor;

/**
 * Save action response.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class SaveActionResponse implements ActionListener {

	ArrayList internalDocuments;

	final private Component superiorComponent;

	// Strings to the target directory
	private String directory;

	// Step to show the time progress
	public final static int HALF_SECOND = 500;
	// Progress bar
	javax.swing.Timer timer;
	ProgressMonitor progressMonitor;

	FileSaver fileSaver;

	/**
	 * Constructor that receives the superior component to allow the usage of
	 * File Open Window and the DefaultListModel to insert the files selected by
	 * the user.
	 */
	SaveActionResponse(Component newSuperiorComponent,
			ArrayList internalDocuments, String directory) {

		superiorComponent = newSuperiorComponent;
		this.internalDocuments = internalDocuments;
		this.directory = directory;
	}

	/**
	 * Print the progress message of files reading
	 */
	class TimerListener implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			if (progressMonitor.isCanceled() || fileSaver.done()) {
				progressMonitor.close();
				fileSaver.stop();
				timer.stop();

			} else {
				progressMonitor.setNote(fileSaver.getMessage());
				progressMonitor.setProgress((int) ((double) fileSaver
						.getCurrent()
						/ (double) fileSaver.getLengthOfTask() * 100.0));
			}
		}
	}

	/**
	 * Insert the files selected in the DefaultListModel
	 */
	public void actionPerformed(ActionEvent e) {

		// Create a document reader to the file names.
		fileSaver = new FileSaver(internalDocuments, directory);

		// Load the files.
		progressMonitor = new ProgressMonitor(superiorComponent, "Loading ",
				"", 0, 100);
		progressMonitor.setProgress(0);
		fileSaver.go();

		timer = new javax.swing.Timer(HALF_SECOND, new TimerListener());

		timer.start();

	}

	/**
	 * Returns the internalDocuments.
	 * 
	 * @return ArrayList
	 */
	public ArrayList getInternalDocuments() {
		return internalDocuments;
	}

}
