/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.stringconcatenation;

import java.util.ArrayList;

import com.carlosbcruz.util.Err;
import com.carlosbcruz.util.FileSupport;
import com.carlosbcruz.util.Out;

/**
 * Helps working with buffers of concatenated fields.
 *  
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class StringConcatenation {

	private ArrayList fields = new ArrayList();

	private static final String DIRECTORY = "C:/";

	private static final boolean LOG_GET_FIELDS = false;
	private static final boolean LOG_READ_PARAMETER = false;
	private static final boolean LOG_GENERATE_STRINGS = false;

	private String fileName;

	/**
	 * Load the structure for the input string for Verazip.
	 */
	private void loadVerazipInputFormat() {

		// Attention. Can not use [, ], <, >, ( or ) on fild name. 

		fileName = "vz2in.txt";

		final int SHIFT = 68;

		fields.add(new Field("A-State Code", 0, 2));
		fields.add(new Field("A-ZIP Code", 2, 5));
		fields.add(new Field("A-City/Province Name", 7, 26));
		fields.add(new Field("A-Canadian Postal Code", 33, 6));
		fields.add(new Field("A-County Name", 39, 26));
		fields.add(new Field("A-County Code", 65, 3));

		fields.add(new Field("B-State Code", SHIFT + 0, 2));
		fields.add(new Field("B-ZIP Code", SHIFT + 2, 5));
		fields.add(new Field("B-City/Province Name", SHIFT + 7, 26));
		fields.add(new Field("B-Canadian Postal Code", SHIFT + 33, 6));
		fields.add(new Field("B-County Name", SHIFT + 39, 26));
		fields.add(new Field("B-County Code", SHIFT + 65, 3));

	}

	/**
	 * Load the structure for the output string from Verazip.
	 */
	private void loadVerazipOutputormat() {

		// Attention. Can not use [, ], <, >, ( or ) on fild name. 

		fileName = "vz2out.txt";

		final int SHIFT_BODY = 208;
		final int SHIFT_SECOND = SHIFT_BODY + 70;

		fields.add(new Field("A-Number of Records Returned", 0, 6));
		fields.add(new Field("A-Completion Code", 6, 2));
		fields.add(new Field("A-Error Message", 8, 200));
		fields.add(new Field("A-State Code", SHIFT_BODY + 0, 2));
		fields.add(new Field("A-First ZIP Code", SHIFT_BODY + 2, 5));
		fields.add(new Field("A-Last ZIP Code", SHIFT_BODY + 7, 5));
		fields.add(new Field("A-Geo Code", SHIFT_BODY + 12, 2));
		fields.add(new Field("A-City/Province Name", SHIFT_BODY + 14, 26));
		fields.add(new Field("A-County Code", SHIFT_BODY + 40, 3));
		fields.add(new Field("A-County Name", SHIFT_BODY + 43, 26));
		fields.add(new Field("A-Outside City Limits Indicator",
				SHIFT_BODY + 69, 1));
		fields.add(new Field("B-Number of Records Returned", SHIFT_SECOND + 0,
				6));
		fields.add(new Field("B-Completion Code", SHIFT_SECOND + 6, 2));
		fields.add(new Field("B-Error Message", SHIFT_SECOND + 8, 200));
		fields.add(new Field("B-State Code", SHIFT_SECOND + SHIFT_BODY + 0, 2));
		fields.add(new Field("B-First ZIP Code", SHIFT_SECOND + SHIFT_BODY + 2,
				5));
		fields.add(new Field("B-Last ZIP Code", SHIFT_SECOND + SHIFT_BODY + 7,
				5));
		fields.add(new Field("B-Geo Code", SHIFT_SECOND + SHIFT_BODY + 12, 2));
		fields.add(new Field("B-City/Province Name", SHIFT_SECOND + SHIFT_BODY
				+ 14, 26));
		fields.add(new Field("B-County Code", SHIFT_SECOND + SHIFT_BODY + 40, 3));
		fields.add(new Field("B-County Name", SHIFT_SECOND + SHIFT_BODY + 43,
				26));
		fields.add(new Field("B-Outside City Limits Indicator", SHIFT_SECOND
				+ SHIFT_BODY + 69, 1));
	}

	/**
	 * Load the structure for the input string for Taxware.
	 */
	private void loadTaxwareInputFormat() {

		fileName = "apo.in";

		final int SHIFT = 60;

		fields.add(new Field("Number of Records In", 0, 6));
		fields.add(new Field("Reserved", 6, SHIFT - 6));

		fields.add(new Field("System Indicator", SHIFT + 0, 1));
		fields.add(new Field("Language Code", SHIFT + 1, 3));
		fields.add(new Field("Company ID", SHIFT + 4, 20));
		fields.add(new Field("Division Code", SHIFT + 24, 20));
		fields.add(new Field("Ship-From County Code", SHIFT + 44, 3));
		fields.add(new Field("Ship-From Territory Code", SHIFT + 47, 3));
		fields.add(new Field("Ship-From State Province", SHIFT + 50, 26));
		fields.add(new Field("Ship-From County Name", SHIFT + 76, 26));
		fields.add(new Field("Ship-From County Code", SHIFT + 102, 3));
		fields.add(new Field("Ship-From City Name", SHIFT + 105, 26));
		fields.add(new Field("Ship-From Postal Code", SHIFT + 131, 9));
		fields.add(new Field("Ship-From ZIP Code Extension", SHIFT + 140, 4));
		fields.add(new Field("Ship-From Geo Code", SHIFT + 144, 2));
		fields.add(new Field("Destination Country Code", SHIFT + 146, 3));
		fields.add(new Field("Destination Territory Code", SHIFT + 149, 3));
		fields.add(new Field("Destination State Province", SHIFT + 152, 26));
		fields.add(new Field("Destination County Name", SHIFT + 178, 26));
		fields.add(new Field("Destination County Code", SHIFT + 204, 3));
		fields.add(new Field("Destination City Name", SHIFT + 207, 26));
		fields.add(new Field("Destination Postal Code", SHIFT + 233, 9));
		fields.add(new Field("Destination ZIP Code Extension", SHIFT + 242, 4));
		fields.add(new Field("Destination Geo Code", SHIFT + 246, 2));
		fields.add(new Field("Destination Sec State Province", SHIFT + 248, 26));
		fields.add(new Field("Destination Sec County Code", SHIFT + 274, 3));
		fields.add(new Field("Destination Sec County Name", SHIFT + 277, 26));
		fields.add(new Field("Destination Sec City", SHIFT + 303, 26));
		fields.add(new Field("Destination Sec Postal Code", SHIFT + 329, 9));
		fields.add(new Field("Destination Sec ZIP Code Extension", SHIFT + 338,
				4));

		fields.add(new Field("Destination Sec Geo Code", SHIFT + 342, 2));
		fields.add(new Field("Origin Country Code", SHIFT + 344, 3));
		fields.add(new Field("Origin Territory Code", SHIFT + 347, 3));
		fields.add(new Field("Origin State/Province", SHIFT + 350, 26));
		fields.add(new Field("Origin County Name", SHIFT + 376, 26));
		fields.add(new Field("Origin County Code", SHIFT + 402, 3));
		fields.add(new Field("Origin City Name", SHIFT + 405, 26));
		fields.add(new Field("Origin Postal Code", SHIFT + 431, 9));
		fields.add(new Field("Origin ZIP Code Extension", SHIFT + 440, 4));
		fields.add(new Field("Origin Geo Code", SHIFT + 444, 2));
		fields.add(new Field("POA Country Code", SHIFT + 446, 3));
		fields.add(new Field("POA Territory Code", SHIFT + 449, 3));
		fields.add(new Field("POA State/Province", SHIFT + 452, 26));
		fields.add(new Field("POA County Name", SHIFT + 478, 26));
		fields.add(new Field("POA County Code", SHIFT + 504, 3));
		fields.add(new Field("POA City Name", SHIFT + 507, 26));
		fields.add(new Field("POA Postal Code", SHIFT + 533, 9));
		fields.add(new Field("POA ZIP Code Extension", SHIFT + 542, 4));
		fields.add(new Field("POA Geo Code", SHIFT + 546, 2));
		fields.add(new Field("Invoiced-To Country Code", SHIFT + 548, 3));
		fields.add(new Field("Invoiced-To Territory Code", SHIFT + 551, 3));
		fields.add(new Field("Invoiced-To State Province", SHIFT + 554, 26));
		fields.add(new Field("Invoiced-To County Name", SHIFT + 580, 26));
		fields.add(new Field("Invoiced-To County Code", SHIFT + 606, 3));
		fields.add(new Field("Invoiced-To City Name", SHIFT + 609, 26));
		fields.add(new Field("Invoiced-To Postal Code", SHIFT + 635, 9));
		fields.add(new Field("Invoiced-To ZIP Code Extension", SHIFT + 644, 4));
		fields.add(new Field("Invoiced-To Geo Code", SHIFT + 648, 2));
		fields.add(new Field("Point of Title Passage", SHIFT + 650, 1));
		fields.add(new Field("Taxing Location", SHIFT + 651, 1));
		fields.add(new Field("Calculation Mode", SHIFT + 652, 1));
		fields.add(new Field("Transaction Type", SHIFT + 653, 1));
		fields.add(new Field("Tax Code/WT Code", SHIFT + 654, 2));
		fields.add(new Field("Type of Tax", SHIFT + 656, 1));
		fields.add(new Field("Invoice Date Tax Point", SHIFT + 657, 8));
		fields.add(new Field("Delivery Date Transaction Date", SHIFT + 665, 8));
		fields.add(new Field("Mode of Transport", SHIFT + 673, 2));
		fields.add(new Field("Commodity Service Code Product Code",
				SHIFT + 675, 25));
		fields.add(new Field("Credit Indicator", SHIFT + 700, 1));
		fields.add(new Field("Reason Code-Country", SHIFT + 701, 2));
		fields.add(new Field("Reason Code-State Province", SHIFT + 703, 2));
		fields.add(new Field("Reason Code-County", SHIFT + 705, 2));
		fields.add(new Field("Reason Code-City", SHIFT + 707, 2));
		fields.add(new Field("Tax Certificate Number-Country", SHIFT + 709, 25));
		fields.add(new Field("Tax Certificate Number-State Province",
				SHIFT + 734, 25));
		fields.add(new Field("Tax Certificate Number-County", SHIFT + 759, 25));
		fields.add(new Field("Tax Certificate Number-City", SHIFT + 784, 25));
		fields.add(new Field("Exempt Indicator-All", SHIFT + 809, 1));
		fields.add(new Field("Exempt Indicator-Country", SHIFT + 810, 1));
		fields.add(new Field("Exempt Indicator-Territory", SHIFT + 811, 1));
		fields.add(new Field("Exempt Indicator-Province", SHIFT + 812, 1));
		fields.add(new Field("Exempt Indicator-County", SHIFT + 813, 1));
		fields.add(new Field("Exempt Indicator-City", SHIFT + 814, 1));
		fields.add(new Field("Exempt Indicator-Secondary State", SHIFT + 815, 1));
		fields.add(new Field("Exempt Indicator-Secondary County", SHIFT + 816,
				1));
		fields.add(new Field("Exempt Indicator-Secondary City", SHIFT + 817, 1));
		fields.add(new Field("Exempt Indicator-District", SHIFT + 818, 1));
		fields.add(new Field("Invoice/Document Number", SHIFT + 819, 20));
		fields.add(new Field("Primary Currency Code", SHIFT + 839, 3));
		fields.add(new Field("Secondary Currency Code", SHIFT + 842, 3));
		fields.add(new Field("Audit File Indicator", SHIFT + 845, 1));
		fields.add(new Field("Exemption Use Flag", SHIFT + 846, 1));
		fields.add(new Field("Exemption Criterion Flag", SHIFT + 847, 1));
		fields.add(new Field("Exemption Process Flag", SHIFT + 848, 1));
		fields.add(new Field("Business/Customer Number", SHIFT + 849, 20));
		fields.add(new Field("Contract/Job Number", SHIFT + 869, 10));
		fields.add(new Field("Cost Center", SHIFT + 879, 10));
		fields.add(new Field("Business Location Code", SHIFT + 889, 13));
		fields.add(new Field("Unit of Measure", SHIFT + 902, 15));
		fields.add(new Field("Description of Item", SHIFT + 917, 100));
		fields.add(new Field("End Invoice Processing", SHIFT + 1017, 1));
		fields.add(new Field("Invoice Summary Indicator", SHIFT + 1018, 1));
		fields.add(new Field("Recoverable Indicator", SHIFT + 1019, 1));
		fields.add(new Field("Reserved", SHIFT + 1020, 1));
		fields.add(new Field("Accounting Reference", SHIFT + 1021, 15));
		fields.add(new Field("Original Document Number", SHIFT + 1036, 20));
		fields.add(new Field("Document Type", SHIFT + 1056, 2));
		fields.add(new Field("Agent Indicator", SHIFT + 1058, 1));
		fields.add(new Field("Agent Branch ID", SHIFT + 1059, 3));
		fields.add(new Field("Country of Origin", SHIFT + 1062, 3));
		fields.add(new Field("Region of Origin/Destination Code", SHIFT + 1065,
				2));
		fields.add(new Field("Delivery Terms", SHIFT + 1067, 5));
		fields.add(new Field("Port of Loading/Unloading", SHIFT + 1072, 5));
		fields.add(new Field("Nature of Transaction Code", SHIFT + 1077, 2));
		fields.add(new Field("Distance Sale Indicator", SHIFT + 1079, 1));
		fields.add(new Field("Tax Free Zone Indicator", SHIFT + 1080, 1));
		fields.add(new Field("Origin Currency Code", SHIFT + 1081, 3));
		fields.add(new Field("Destination Currency Code", SHIFT + 1084, 3));
		fields.add(new Field("Statistical Procedure", SHIFT + 1087, 6));
		fields.add(new Field("Supplementary Units", SHIFT + 1093, 11));
		fields.add(new Field("Corrections Code", SHIFT + 1104, 1));
		fields.add(new Field("Reverse Charge Indicator", SHIFT + 1105, 1));
		fields.add(new Field("New Means of Transportation Indicator",
				SHIFT + 1106, 1));
		fields.add(new Field("BOE Code", SHIFT + 1107, 2));
		fields.add(new Field("Affiliation", SHIFT + 1109, 1));
		fields.add(new Field("CSA", SHIFT + 1110, 1));
		fields.add(new Field("Line Item Cost Indicator", SHIFT + 1111, 1));
		fields.add(new Field("Drop Shipment Indicator", SHIFT + 1112, 1));
		fields.add(new Field("Customer Name", SHIFT + 1113, 20));
		fields.add(new Field("Taxsel Parm", SHIFT + 1133, 1));
		fields.add(new Field("Part Number", SHIFT + 1134, 20));
		fields.add(new Field("Fiscal Date", SHIFT + 1154, 8));
		fields.add(new Field("Miscellaneous Information", SHIFT + 1162, 50));
		fields.add(new Field("UEZ Flag", SHIFT + 1212, 1));
		fields.add(new Field("Extra Indicator 1", SHIFT + 1213, 1));
		fields.add(new Field("Extra Indicator 2", SHIFT + 1214, 1));
		fields.add(new Field("Extra Indicator 3", SHIFT + 1215, 1));
		fields.add(new Field("Audit File Type", SHIFT + 1216, 1));
		fields.add(new Field("Bill-To Customer Name", SHIFT + 1217, 20));
		fields.add(new Field("Bill-To Customer ID", SHIFT + 1237, 20));
		fields.add(new Field("Service Indicator", SHIFT + 1257, 1));
		fields.add(new Field("Rounding Indicator", SHIFT + 1258, 1));
		fields.add(new Field("General Indicator", SHIFT + 1259, 1));
		fields.add(new Field("Movement Code", SHIFT + 1260, 1));
		fields.add(new Field("Storage Code", SHIFT + 1261, 1));
		fields.add(new Field("Product Code Conversion", SHIFT + 1262, 1));
		fields.add(new Field("Product Code Type", SHIFT + 1263, 1));
		fields.add(new Field("Federal Tax Type", SHIFT + 1264, 1));
		fields.add(new Field("Territory Tax Type", SHIFT + 1265, 1));
		fields.add(new Field("State Tax Type", SHIFT + 1266, 1));
		fields.add(new Field("County Tax Type", SHIFT + 1267, 1));
		fields.add(new Field("City Tax Type", SHIFT + 1268, 1));
		fields.add(new Field("Secondary State Tax Type", SHIFT + 1269, 1));
		fields.add(new Field("Secondary County Tax Type", SHIFT + 1270, 1));
		fields.add(new Field("Secondary City Tax Type", SHIFT + 1271, 1));
		fields.add(new Field("District Tax Type", SHIFT + 1272, 1));
		fields.add(new Field("No Tax Indicator", SHIFT + 1273, 1));
		fields.add(new Field("Federal No Tax Indicator", SHIFT + 1274, 1));
		fields.add(new Field("Territory No Tax Indicator", SHIFT + 1275, 1));
		fields.add(new Field("State/Province No Tax Indicator", SHIFT + 1276, 1));
		fields.add(new Field("County No Tax Indicator", SHIFT + 1277, 1));
		fields.add(new Field("City No Tax Indicator", SHIFT + 1278, 1));
		fields.add(new Field("Secondary County No Tax Indicator", SHIFT + 1279,
				1));
		fields.add(new Field("Secondary City No Tax Indicator", SHIFT + 1280, 1));
		fields.add(new Field("Secondary State/Province No Tax Indicator",
				SHIFT + 1281, 1));
		fields.add(new Field("District No Tax Indicator", SHIFT + 1282, 1));
		fields.add(new Field("Inside/Outside City Limits Code", SHIFT + 1283, 1));
		fields.add(new Field("Ship-To/Outside City Limits Code", SHIFT + 1284,
				1));
		fields.add(new Field("Ship-From/Outside City Limits Code",
				SHIFT + 1285, 1));
		fields.add(new Field("POO Outside City Limits Code", SHIFT + 1286, 1));
		fields.add(new Field("POA Outside City Limits Code", SHIFT + 1287, 1));
		fields.add(new Field("Bill-To Inside/Outside City Limits Code",
				SHIFT + 1288, 1));
		fields.add(new Field("Ship-To Location Place of Business",
				SHIFT + 1289, 1));
		fields.add(new Field("Ship-From Location Place of Business",
				SHIFT + 1290, 1));
		fields.add(new Field("POO Place of Business", SHIFT + 1291, 1));
		fields.add(new Field("POA Place of Business", SHIFT + 1292, 1));
		fields.add(new Field("Short City Name Indicator", SHIFT + 1293, 1));
		fields.add(new Field("Seller Registration Number", SHIFT + 1294, 25));
		fields.add(new Field("Buyer Registration Number", SHIFT + 1319, 25));
		fields.add(new Field("Report Exclusion Indicator", SHIFT + 1344, 1));
		fields.add(new Field("Work Order Number", SHIFT + 1345, 26));
		fields.add(new Field("Facility SF Code", SHIFT + 1371, 2));
		fields.add(new Field("Facility State Code", SHIFT + 1373, 2));
		fields.add(new Field("Accounting Reference Extension", SHIFT + 1375, 1));
		fields.add(new Field("Bill to Customer ID Extension", SHIFT + 1376, 5));
		fields.add(new Field("Document Number Extension", SHIFT + 1381, 3));
		fields.add(new Field("Terminal Location Code", SHIFT + 1384, 13));
		fields.add(new Field("License NumberCountry", SHIFT + 1397, 3));
		fields.add(new Field("License NumberProvince", SHIFT + 1400, 3));
		fields.add(new Field("License NumberCounty", SHIFT + 1403, 3));
		fields.add(new Field("License NumberCity", SHIFT + 1406, 3));
		fields.add(new Field("Odometer Reading", SHIFT + 1409, 7));
		fields.add(new Field("Start Date", SHIFT + 1416, 8));
		fields.add(new Field("End Date", SHIFT + 1424, 8));
		fields.add(new Field("Agent Registration Number", SHIFT + 1432, 25));
		fields.add(new Field("Reserved 2", SHIFT + 1457, 178));
		fields.add(new Field("Fee 1 Override ID", SHIFT + 1635, 5));
		fields.add(new Field("Fee 1 Tax Type", SHIFT + 1640, 1));
		fields.add(new Field("Fee 1 No Tax Indicator", SHIFT + 1641, 1));
		fields.add(new Field("Exempt Indicator - Fee 1", SHIFT + 1642, 1));
		fields.add(new Field("Exempt Amount - Fee 1", SHIFT + 1643, 14));
		fields.add(new Field("Override Fee 1 Amount", SHIFT + 1657, 14));
		fields.add(new Field("Override Fee 1 Rate", SHIFT + 1671, 7));
		fields.add(new Field("Fee 2 Override ID", SHIFT + 1678, 5));
		fields.add(new Field("Fee 2 Tax Type", SHIFT + 1683, 1));
		fields.add(new Field("Fee 2 No Tax Indicator", SHIFT + 1684, 1));
		fields.add(new Field("Exempt Indicator - Fee 2", SHIFT + 1685, 1));
		fields.add(new Field("Exempt Amount - Fee 2", SHIFT + 1686, 14));
		fields.add(new Field("Override Fee 2 Amount", SHIFT + 1700, 14));
		fields.add(new Field("Override Fee 2 Rate", SHIFT + 1714, 7));
		fields.add(new Field("Fee 3 Override ID", SHIFT + 1721, 5));
		fields.add(new Field("Fee 3 Tax Type", SHIFT + 1726, 1));
		fields.add(new Field("Fee 3 No Tax Indicator", SHIFT + 1727, 1));
		fields.add(new Field("Exempt Indicator - Fee 3", SHIFT + 1728, 1));
		fields.add(new Field("Exempt Amount - Fee 3", SHIFT + 1729, 14));
		fields.add(new Field("Override Fee 3 Amount", SHIFT + 1743, 14));
		fields.add(new Field("Override Fee 3 Rate", SHIFT + 1757, 7));
		fields.add(new Field("Fee 4 Override ID", SHIFT + 1764, 5));
		fields.add(new Field("Fee 4 Tax Type", SHIFT + 1769, 1));
		fields.add(new Field("Fee 4 No Tax Indicator", SHIFT + 1770, 1));
		fields.add(new Field("Exempt Indicator - Fee 4", SHIFT + 1771, 1));
		fields.add(new Field("Exempt Amount - Fee 4", SHIFT + 1772, 14));
		fields.add(new Field("Override Fee 4 Amount", SHIFT + 1786, 14));
		fields.add(new Field("Override Fee 4 Rate", SHIFT + 1800, 7));
		fields.add(new Field("Fee 5 Override ID", SHIFT + 1807, 5));
		fields.add(new Field("Fee 5 Tax Type", SHIFT + 1812, 1));
		fields.add(new Field("Fee 5 No Tax Indicator", SHIFT + 1813, 1));
		fields.add(new Field("Exempt Indicator - Fee 5", SHIFT + 1814, 1));
		fields.add(new Field("Exempt Amount - Fee 5", SHIFT + 1815, 14));
		fields.add(new Field("Override Fee 5 Amount", SHIFT + 1829, 14));
		fields.add(new Field("Override Fee 5 Rate", SHIFT + 1843, 7));
		fields.add(new Field("Tax Certificate Number - Fees", SHIFT + 1850, 25));
		fields.add(new Field("Manufacturers Coupon", SHIFT + 1875, 14));
		fields.add(new Field("Retailers Coupon", SHIFT + 1889, 14));
		fields.add(new Field("Basis Percentage Country2", SHIFT + 1903, 7));
		fields.add(new Field("Exempt Amount Country2", SHIFT + 1910, 14));
		fields.add(new Field("Reason Code Country2", SHIFT + 1924, 2));
		fields.add(new Field("Exempt Country2", SHIFT + 1926, 1));
		fields.add(new Field("Override Rate Country2", SHIFT + 1927, 7));
		fields.add(new Field("Override Amount Country2", SHIFT + 1934, 14));
		fields.add(new Field("Recoverable Percentage Country2", SHIFT + 1948, 7));
		fields.add(new Field("Carrier Name 1", SHIFT + 1955, 26));
		fields.add(new Field("Carrier Name 2", SHIFT + 1981, 9));
		fields.add(new Field("Vendor Name 1", SHIFT + 1990, 26));
		fields.add(new Field("Vendor Name 2", SHIFT + 2016, 9));
		fields.add(new Field("Transaction Time", SHIFT + 2025, 4));
		fields.add(new Field("Gross Volume", SHIFT + 2029, 15));
		fields.add(new Field("Override Tax Amount-Country", SHIFT + 2044, 14));
		fields.add(new Field("Override Tax Amount-Territory", SHIFT + 2058, 14));
		fields.add(new Field("Override Tax Amount-State/Province",
				SHIFT + 2072, 14));
		fields.add(new Field("Override Tax Amount-County", SHIFT + 2086, 14));
		fields.add(new Field("Override Tax Amount-City", SHIFT + 2100, 14));
		fields.add(new Field("Override Tax Amount-District", SHIFT + 2114, 14));
		fields.add(new Field("Override Tax Amount-Sec State/Province",
				SHIFT + 2128, 14));
		fields.add(new Field("Override Tax Amount-Sec County", SHIFT + 2142, 14));
		fields.add(new Field("Override Tax Amount-Sec City/Town", SHIFT + 2156,
				14));
		fields.add(new Field("Override Rate-Country", SHIFT + 2170, 7));
		fields.add(new Field("Override Rate-Territory", SHIFT + 2177, 7));
		fields.add(new Field("Override Rate-State/Province", SHIFT + 2184, 7));
		fields.add(new Field("Override Rate-County", SHIFT + 2191, 7));
		fields.add(new Field("Override Rate-City", SHIFT + 2198, 7));
		fields.add(new Field("Override Rate-District", SHIFT + 2205, 7));
		fields.add(new Field("Override Rate-Sec State", SHIFT + 2212, 7));
		fields.add(new Field("Override Rate-Sec County", SHIFT + 2219, 7));
		fields.add(new Field("Override Rate-Sec City", SHIFT + 2226, 7));
		fields.add(new Field("Number of Items", SHIFT + 2233, 7));
		fields.add(new Field("Invoice Line Number", SHIFT + 2240, 5));
		fields.add(new Field("Line Item Amount", SHIFT + 2245, 14));
		fields.add(new Field("Tax Amount", SHIFT + 2259, 14));
		fields.add(new Field("Discount Amount", SHIFT + 2273, 14));
		fields.add(new Field("Freight Amount", SHIFT + 2287, 14));
		fields.add(new Field("Insurance Amount-Local", SHIFT + 2301, 14));
		fields.add(new Field("Exempt Amount-Country", SHIFT + 2315, 14));
		fields.add(new Field("Exempt Amount-Territory", SHIFT + 2329, 14));
		fields.add(new Field("Exempt Amount-State/Province", SHIFT + 2343, 14));
		fields.add(new Field("Exempt Amount-County", SHIFT + 2357, 14));
		fields.add(new Field("Exempt Amount-City", SHIFT + 2371, 14));
		fields.add(new Field("Exempt Amount-District", SHIFT + 2385, 14));
		fields.add(new Field("Exempt Amount-Sec State", SHIFT + 2399, 14));
		fields.add(new Field("Exempt Amount-Sec County", SHIFT + 2413, 14));
		fields.add(new Field("Exempt Amount-Sec City", SHIFT + 2427, 14));
		fields.add(new Field("Insurance Amount-Foreign", SHIFT + 2441, 14));
		fields.add(new Field("Shipping Amount-Foreign", SHIFT + 2455, 14));
		fields.add(new Field("Contract Amount", SHIFT + 2469, 14));
		fields.add(new Field("Install Amount", SHIFT + 2483, 14));
		fields.add(new Field("Basis Percentage", SHIFT + 2497, 7));
		fields.add(new Field("Currency Conversion Factor", SHIFT + 2504, 15));
		fields.add(new Field("Net Mass/Volume", SHIFT + 2519, 15));
		fields.add(new Field("Ship-From Currency Conversion Factor",
				SHIFT + 2534, 15));
		fields.add(new Field("Destination Currency Conversion Factor",
				SHIFT + 2549, 15));
		fields.add(new Field("Recoverable Percentage", SHIFT + 2564, 7));
		fields.add(new Field("Exponent", SHIFT + 2571, 3));
		fields.add(new Field("Precision Factor Line Item Amount", SHIFT + 2574,
				3));
		fields.add(new Field("Override Amount Precision Factor-Country",
				SHIFT + 2577, 3));
		fields.add(new Field("Override Amount Precision Factor - Territory",
				SHIFT + 2580, 3));
		fields.add(new Field(
				"Override Amount Precision Factor - State/Province",
				SHIFT + 2583, 3));
		fields.add(new Field("Override Amount Precision Factor - County",
				SHIFT + 2586, 3));
		fields.add(new Field("Override Amount Precision Factor - City",
				SHIFT + 2589, 3));
	}

	/**
	 * Load the structure for the output string for Taxware.
	 */
	private void loadTaxwareOutputFormat() {

		fileName = "apo.txt";

		final int SHIFT = 284;

		fields.add(new Field("Number of Records Out", 0, 6));
		fields.add(new Field("Common Completion Code", 6, 4));
		fields.add(new Field("Completion Code Description", 10, 200));
		fields.add(new Field("Merchant Name", 210, 20));
		fields.add(new Field("Compression Indicator", 230, 1));
		fields.add(new Field("Compression Character 1", 231, 1));
		fields.add(new Field("Compression Character 2", 232, 1));
		fields.add(new Field("Open/Close Processing Indicator", 233, 1));
		fields.add(new Field("Reserved", 234, 50));

		fields.add(new Field("System Indicator", SHIFT + 0, 1));
		fields.add(new Field("General Completion Code", SHIFT + 1, 4));
		fields.add(new Field("Federal Completion Code", SHIFT + 5, 4));
		fields.add(new Field("Territory Completion Code", SHIFT + 9, 4));
		fields.add(new Field("State/Province Completion Code", SHIFT + 13, 4));
		fields.add(new Field("County Completion Code", SHIFT + 17, 4));
		fields.add(new Field("City Completion Code", SHIFT + 21, 4));
		fields.add(new Field("Secondary State/Province Completion Code",
				SHIFT + 25, 4));
		fields.add(new Field("Secondary County Completion Code", SHIFT + 29, 4));
		fields.add(new Field("Secondary City Completion Code", SHIFT + 33, 4));
		fields.add(new Field("District Completion Code", SHIFT + 37, 4));
		fields.add(new Field("Extra Completion Code 1", SHIFT + 41, 4));
		fields.add(new Field("Extra Completion Code 2", SHIFT + 45, 4));
		fields.add(new Field("Extra Completion Code 3", SHIFT + 49, 4));
		fields.add(new Field("Extra Completion Code 4", SHIFT + 53, 4));
		fields.add(new Field("Suspense Code - Country", SHIFT + 57, 4));
		fields.add(new Field("Suspense Code - Province", SHIFT + 61, 4));
		fields.add(new Field("Suspense Code - County", SHIFT + 65, 4));
		fields.add(new Field("Suspense Code - City", SHIFT + 69, 4));
		fields.add(new Field("General Completion Code Description", SHIFT + 73,
				200));
		fields.add(new Field("System Date", SHIFT + 273, 8));
		fields.add(new Field("Transaction Number", SHIFT + 281, 10));
		fields.add(new Field("Line Item ID", SHIFT + 291, 5));
		fields.add(new Field("Manual Transaction Indicator", SHIFT + 296, 1));
		fields.add(new Field("Zero Rated Indicator", SHIFT + 297, 1));
		fields.add(new Field("Jurisdiction / Taxing Location Type",
				SHIFT + 298, 1));
		fields.add(new Field("Country of Jurisdiction Code", SHIFT + 299, 3));
		fields.add(new Field("Territory of Jurisdiction Code", SHIFT + 302, 3));
		fields.add(new Field("State/Province of Jurisdiction", SHIFT + 305, 26));
		fields.add(new Field("County Code of Jurisdiction", SHIFT + 331, 3));
		fields.add(new Field("City of Jurisdiction", SHIFT + 334, 26));
		fields.add(new Field("ZIP Code of Jurisdiction", SHIFT + 360, 9));
		fields.add(new Field("ZIP Code Extension of Jurisdiction", SHIFT + 369,
				4));
		fields.add(new Field("Geo Code of Jurisdiction", SHIFT + 373, 2));
		fields.add(new Field("Sec State of Jurisdiction", SHIFT + 375, 26));
		fields.add(new Field("Sec County of Jurisdiction", SHIFT + 401, 3));
		fields.add(new Field("Sec City of Jurisdiction", SHIFT + 404, 26));
		fields.add(new Field("Sec ZIP Code of Jurisdiction", SHIFT + 430, 9));
		fields.add(new Field("Sec ZIP Code Extension of Jurisdiction",
				SHIFT + 439, 4));
		fields.add(new Field("Sec Geo Code of Jurisdiction", SHIFT + 443, 2));
		fields.add(new Field("Federal Tax Type", SHIFT + 445, 1));
		fields.add(new Field("State Tax Type", SHIFT + 446, 1));
		fields.add(new Field("County Tax Type", SHIFT + 447, 1));
		fields.add(new Field("City Tax Type", SHIFT + 448, 1));
		fields.add(new Field("Sec State Tax Type", SHIFT + 449, 1));
		fields.add(new Field("Sec County Tax Type", SHIFT + 450, 1));
		fields.add(new Field("Sec City Tax Type", SHIFT + 451, 1));
		fields.add(new Field("District Tax Type", SHIFT + 452, 1));
		fields.add(new Field("Reason Code - Country", SHIFT + 453, 2));
		fields.add(new Field("Reason Code - State/Province", SHIFT + 455, 2));
		fields.add(new Field("Reason Code - County", SHIFT + 457, 2));
		fields.add(new Field("Reason Code - City", SHIFT + 459, 2));
		fields.add(new Field("Tax Certificate - Country", SHIFT + 461, 25));
		fields.add(new Field("Tax Certificate - State/Province", SHIFT + 486,
				25));
		fields.add(new Field("Tax Certificate - County", SHIFT + 511, 25));
		fields.add(new Field("Tax Certificate - City", SHIFT + 536, 25));
		fields.add(new Field("STEP Country Status Code", SHIFT + 561, 1));
		fields.add(new Field("STEP Country Comment Code", SHIFT + 562, 1));
		fields.add(new Field("STEP State/Province Status Code", SHIFT + 563, 1));
		fields.add(new Field("STEP State/Province Comment Code", SHIFT + 564, 1));
		fields.add(new Field("STEP County Status Code", SHIFT + 565, 1));
		fields.add(new Field("STEP County Comment Code", SHIFT + 566, 1));
		fields.add(new Field("STEP City Status Code", SHIFT + 567, 1));
		fields.add(new Field("STEP City Comment Code", SHIFT + 568, 1));
		fields.add(new Field("Jurisdiction County Name", SHIFT + 569, 26));
		fields.add(new Field("Secondary Jurisdiction County Name", SHIFT + 595,
				26));
		fields.add(new Field("Seller Registration Number", SHIFT + 621, 25));
		fields.add(new Field("Reserved2", SHIFT + 646, 167));
		fields.add(new Field("Fee 1 ID", SHIFT + 814, 5));
		fields.add(new Field("Fee 1 Completion Code", SHIFT + 819, 4));
		fields.add(new Field("STEP Comment Code Fee 1", SHIFT + 823, 1));
		fields.add(new Field("STEP Status Code Fee 1", SHIFT + 824, 1));
		fields.add(new Field("Fee 1 Tax Type", SHIFT + 825, 1));
		fields.add(new Field("Calculated Amount - Fee 1", SHIFT + 826, 14));
		fields.add(new Field("Basis Amount - Fee 1", SHIFT + 840, 14));
		fields.add(new Field("Fee 1 Exempt Amount", SHIFT + 854, 14));
		fields.add(new Field("Rate - Fee 1", SHIFT + 868, 7));
		fields.add(new Field("Fee 1 Short Description", SHIFT + 875, 25));
		fields.add(new Field("Fee 2 ID", SHIFT + 900, 5));
		fields.add(new Field("Fee 2 Completion Code", SHIFT + 905, 4));
		fields.add(new Field("STEP Comment Code Fee 2", SHIFT + 909, 1));
		fields.add(new Field("STEP Status Code Fee 2", SHIFT + 910, 1));
		fields.add(new Field("Fee 2 Tax Type", SHIFT + 911, 1));
		fields.add(new Field("Calculated Amount - Fee 2", SHIFT + 912, 14));
		fields.add(new Field("Basis Amount - Fee 2", SHIFT + 926, 14));
		fields.add(new Field("Fee 2 Exempt Amount", SHIFT + 940, 14));
		fields.add(new Field("Rate - Fee 2", SHIFT + 954, 7));
		fields.add(new Field("Fee 2 Short Description", SHIFT + 961, 25));
		fields.add(new Field("Fee 3 ID", SHIFT + 986, 5));
		fields.add(new Field("Fee 3 Completion Code", SHIFT + 991, 4));
		fields.add(new Field("STEP Comment Code Fee 3", SHIFT + 995, 1));
		fields.add(new Field("STEP Status Code Fee 3", SHIFT + 996, 1));
		fields.add(new Field("Fee 3 Tax Type", SHIFT + 997, 1));
		fields.add(new Field("Calculated Amount - Fee 3", SHIFT + 998, 14));
		fields.add(new Field("Basis Amount - Fee 3", SHIFT + 1012, 14));
		fields.add(new Field("Fee 3 Exempt Amount", SHIFT + 1026, 14));
		fields.add(new Field("Rate - Fee 3", SHIFT + 1040, 7));
		fields.add(new Field("Fee 3 Short Description", SHIFT + 1047, 25));
		fields.add(new Field("Fee 4 ID", SHIFT + 1072, 5));
		fields.add(new Field("Fee 4 Completion Code", SHIFT + 1077, 4));
		fields.add(new Field("STEP Comment Code Fee 4", SHIFT + 1081, 1));
		fields.add(new Field("STEP Status Code Fee 4", SHIFT + 1082, 1));
		fields.add(new Field("Fee 4 Tax Type", SHIFT + 1083, 1));
		fields.add(new Field("Calculated Amount - Fee 4", SHIFT + 1084, 14));
		fields.add(new Field("Basis Amount - Fee 4", SHIFT + 1098, 14));
		fields.add(new Field("Fee 4 Exempt Amount", SHIFT + 1112, 14));
		fields.add(new Field("Rate - Fee 4", SHIFT + 1126, 7));
		fields.add(new Field("Fee 4 Short Description", SHIFT + 1133, 25));
		fields.add(new Field("Fee 5 ID", SHIFT + 1158, 5));
		fields.add(new Field("Fee 5 Completion Code", SHIFT + 1163, 4));
		fields.add(new Field("STEP Comment Code Fee 5", SHIFT + 1167, 1));
		fields.add(new Field("STEP Status Code Fee 5", SHIFT + 1168, 1));
		fields.add(new Field("Fee 5 Tax Type", SHIFT + 1169, 1));
		fields.add(new Field("Calculated Amount - Fee 5", SHIFT + 1170, 14));
		fields.add(new Field("Basis Amount - Fee 5", SHIFT + 1184, 14));
		fields.add(new Field("Fee 5 Exempt Amount", SHIFT + 1198, 14));
		fields.add(new Field("Rate - Fee 5", SHIFT + 1212, 7));
		fields.add(new Field("Fee 5 Short Description", SHIFT + 1219, 25));
		fields.add(new Field("Manufacturers Coupon Flag", SHIFT + 1244, 1));
		fields.add(new Field("Retailers Coupon Flag", SHIFT + 1245, 1));
		fields.add(new Field("Discount Coupon Flag", SHIFT + 1246, 1));
		fields.add(new Field("Calculated Tax Amount Country2", SHIFT + 1247, 14));
		fields.add(new Field("Zero Rated Indicator2", SHIFT + 1261, 1));
		fields.add(new Field("Recoverable Percentage Country2", SHIFT + 1262, 7));
		fields.add(new Field("Calculated Tax Rate Country2", SHIFT + 1269, 7));
		fields.add(new Field("Completion Code Country2", SHIFT + 1276, 4));
		fields.add(new Field("Total Value Product", SHIFT + 1280, 14));
		fields.add(new Field("Total Value Invoice", SHIFT + 1294, 14));
		fields.add(new Field("Tax Rate Country2", SHIFT + 1308, 7));
		fields.add(new Field("Suspense Code Country2", SHIFT + 1315, 4));
		fields.add(new Field("Line Item Amount", SHIFT + 1319, 14));
		fields.add(new Field("Calculated Tax Amount - Country", SHIFT + 1333,
				14));
		fields.add(new Field("Calculated Tax Amount - Territory", SHIFT + 1347,
				14));
		fields.add(new Field("Calculated Tax Amount - State/Province",
				SHIFT + 1361, 14));
		fields.add(new Field("Calculated Tax Amount - County", SHIFT + 1375, 14));
		fields.add(new Field("Calculated Tax Amount - City", SHIFT + 1389, 14));
		fields.add(new Field("Calculated Tax Amount - Sec State/Province",
				SHIFT + 1403, 14));
		fields.add(new Field("Calculated Tax Amount - Sec County",
				SHIFT + 1417, 14));
		fields.add(new Field("Calculated Tax Amount - Sec City", SHIFT + 1431,
				14));
		fields.add(new Field("Calculated Tax Amount - District", SHIFT + 1445,
				14));
		fields.add(new Field("Basis Amount- Country", SHIFT + 1459, 14));
		fields.add(new Field("Basis Amount- Territory", SHIFT + 1473, 14));
		fields.add(new Field("Basis Amount - State/Province", SHIFT + 1487, 14));
		fields.add(new Field("Basis Amount - County", SHIFT + 1501, 14));
		fields.add(new Field("Basis Amount - City", SHIFT + 1515, 14));
		fields.add(new Field("Basis Amount - Sec State/Province", SHIFT + 1529,
				14));
		fields.add(new Field("Basis Amount - Sec County", SHIFT + 1543, 14));
		fields.add(new Field("Basis Amount - Sec City", SHIFT + 1557, 14));
		fields.add(new Field("Basis Amount - District", SHIFT + 1571, 14));
		fields.add(new Field("Tax Rate - Country", SHIFT + 1585, 7));
		fields.add(new Field("Tax Rate - Territory", SHIFT + 1592, 7));
		fields.add(new Field("Tax Rate - State/Province", SHIFT + 1599, 7));
		fields.add(new Field("Tax Rate - County", SHIFT + 1606, 7));
		fields.add(new Field("Tax Rate - City", SHIFT + 1613, 7));
		fields.add(new Field("Tax Rate - Sec State/Province", SHIFT + 1620, 7));
		fields.add(new Field("Tax Rate - Sec County", SHIFT + 1627, 7));
		fields.add(new Field("Tax Rate - Sec City", SHIFT + 1634, 7));
		fields.add(new Field("Tax Rate - District", SHIFT + 1641, 7));
		fields.add(new Field("Calculated Tax Rate - Country", SHIFT + 1648, 7));
		fields.add(new Field("Calculated Tax Rate - Territory", SHIFT + 1655, 7));
		fields.add(new Field("Calculated Tax Rate - Province", SHIFT + 1662, 7));
		fields.add(new Field("Calculated Tax Rate - County", SHIFT + 1669, 7));
		fields.add(new Field("Calculated Tax Rate - City", SHIFT + 1676, 7));
		fields.add(new Field("Report Indicator", SHIFT + 1683, 6));
		fields.add(new Field("Calc Tax Amount Precision Factor - Country",
				SHIFT + 1689, 3));
		fields.add(new Field("Calc Tax Amount Precision Factor - Territory",
				SHIFT + 1692, 3));
		fields.add(new Field("Calc Tax Amount Precision Factor - Province",
				SHIFT + 1695, 3));
		fields.add(new Field("Calc Tax Amount Precision Factor - County",
				SHIFT + 1698, 3));
		fields.add(new Field("Calc Tax Amount Precision Factor - City",
				SHIFT + 1701, 3));

	}

	/**
	 * Show the required fields
	 */
	private void showRequiredFields() {

		Field field = null;

		System.out.println("");
		System.out.println("Required Fields:");
		System.out.println("");

		if ("1".equals(getFieldValue("Taxsel Parm"))) {

			System.out
					.println("The following information is required for determining jurisdiction. (Taxsel Parm fieldcontains a 1.)");

			System.out.println("");
			System.out.println(" Point-of-Order Acceptance (POA)");
			field = getField("POA Country Code");
			showField(field);
			field = getField("POA Territory Code");
			showField(field);
			field = getField("POA State/Province");
			showField(field);
			field = getField("POA County Name");
			showField(field);
			field = getField("POA County Code");
			showField(field);
			field = getField("POA City Name");
			showField(field);
			field = getField("POA Postal Code");
			showField(field);
			field = getField("POA ZIP Code Extension");
			showField(field);
			field = getField("POA Geo Code");
			showField(field);

			System.out.println("");
			System.out.println(" Point-of-Order Origin (POO)");
			field = getField("Origin Country Code");
			showField(field);
			field = getField("Origin Territory Code");
			showField(field);
			field = getField("Origin State/Province");
			showField(field);
			field = getField("Origin County Name");
			showField(field);
			field = getField("Origin County Code");
			showField(field);
			field = getField("Origin City Name");
			showField(field);
			field = getField("Origin Postal Code");
			showField(field);
			field = getField("Origin ZIP Code Extension");
			showField(field);
			field = getField("Origin Geo Code");
			showField(field);

			System.out.println("");
			System.out
					.println(" Ship-From Location (including Ship-From Country Code)");
			field = getField("Ship-From County Code");
			showField(field);

			field = getField("Ship-From Territory Code");
			showField(field);

			field = getField("Ship-From State Province");
			showField(field);

			field = getField("Ship-From County Name");
			showField(field);

			field = getField("Ship-From County Code");
			showField(field);

			field = getField("Ship-From City Name");
			showField(field);

			field = getField("Ship-From Postal Code");
			showField(field);

			field = getField("Ship-From ZIP Code Extension");
			showField(field);

			field = getField("Ship-From Geo Code");
			showField(field);

			System.out.println("");
			System.out
					.println(" Ship-To Location (Destination, including Ship-To Country Code)");
			field = getField("Destination Country Code");
			showField(field);

			field = getField("Destination Territory Code");
			showField(field);

			field = getField("Destination State Province");
			showField(field);

			field = getField("Destination County Name");
			showField(field);

			field = getField("Destination County Code");
			showField(field);

			field = getField("Destination City Name");
			showField(field);

			field = getField("Destination Postal Code");
			showField(field);

			field = getField("Destination ZIP Code Extension");
			showField(field);

			field = getField("Destination Geo Code");
			showField(field);

			field = getField("Destination Geo Code");
			showField(field);

			field = getField("Destination Sec State Province");
			showField(field);

			field = getField("Destination Sec County Code");
			showField(field);

			field = getField("Destination Sec County Name");
			showField(field);

			field = getField("Destination Sec City");
			showField(field);

			field = getField("Destination Sec Postal Code");
			showField(field);

			field = getField("Destination Sec ZIP Code Extension");
			showField(field);

			field = getField("Destination Sec Geo Code");
			showField(field);

		}

		if ("2".equals(getFieldValue("Taxsel Parm"))) {
			System.out
					.println("The following information is required for calculating taxes. (Taxsel Parm field contains a 2 or space.)");

			System.out.println(" Company ID");
			field = getField("Company ID");
			showField(field);

			System.out.println(" Destination State Code");
			field = getField("Destination State Province");
			showField(field);

			System.out.println(" Destination ZIP Code");
			field = getField("Destination Postal Code");
			showField(field);
			field = getField("Destination ZIP Code Extension");
			showField(field);

			System.out.println(" An amount (gross or tax)");
			field = getField("Line Item Amount");
			showField(field);
		}

		if ("3".equals(getFieldValue("Taxsel Parm"))) {

			System.out
					.println("The following information is required to determine jurisdiction and calculate taxes at the same time. (Taxsel Parm field contains a 3.)");

			System.out.println("");
			System.out.println(" Point-of-Order Acceptance (POA)");
			field = getField("POA Country Code");
			showField(field);
			field = getField("POA Territory Code");
			showField(field);
			field = getField("POA State/Province");
			showField(field);
			field = getField("POA County Name");
			showField(field);
			field = getField("POA County Code");
			showField(field);
			field = getField("POA City Name");
			showField(field);
			field = getField("POA Postal Code");
			showField(field);
			field = getField("POA ZIP Code Extension");
			showField(field);
			field = getField("POA Geo Code");
			showField(field);

			System.out.println("");
			System.out.println(" Point-of-Order Origin (POO)");

			field = getField("Origin Country Code");
			showField(field);
			field = getField("Origin Territory Code");
			showField(field);
			field = getField("Origin State/Province");
			showField(field);
			field = getField("Origin County Name");
			showField(field);
			field = getField("Origin County Code");
			showField(field);
			field = getField("Origin City Name");
			showField(field);
			field = getField("Origin Postal Code");
			showField(field);
			field = getField("Origin ZIP Code Extension");
			showField(field);
			field = getField("Origin Geo Code");
			showField(field);

			System.out.println("");
			System.out
					.println(" Ship-From Location (including Ship-From Country Code)");

			field = getField("Ship-From County Code");
			showField(field);

			field = getField("Ship-From Territory Code");
			showField(field);

			field = getField("Ship-From State Province");
			showField(field);

			field = getField("Ship-From County Name");
			showField(field);

			field = getField("Ship-From County Code");
			showField(field);

			field = getField("Ship-From City Name");
			showField(field);

			field = getField("Ship-From Postal Code");
			showField(field);

			field = getField("Ship-From ZIP Code Extension");
			showField(field);

			field = getField("Ship-From Geo Code");
			showField(field);

			System.out.println("");
			System.out
					.println(" Ship-To Location (Destination, including Ship-To Country Code)");

			field = getField("Destination Country Code");
			showField(field);

			field = getField("Destination Territory Code");
			showField(field);

			field = getField("Destination State Province");
			showField(field);

			field = getField("Destination County Name");
			showField(field);

			field = getField("Destination County Code");
			showField(field);

			field = getField("Destination City Name");
			showField(field);

			field = getField("Destination Postal Code");
			showField(field);

			field = getField("Destination ZIP Code Extension");
			showField(field);

			field = getField("Destination Geo Code");
			showField(field);

			field = getField("Destination Geo Code");
			showField(field);

			field = getField("Destination Sec State Province");
			showField(field);

			field = getField("Destination Sec County Code");
			showField(field);

			field = getField("Destination Sec County Name");
			showField(field);

			field = getField("Destination Sec City");
			showField(field);

			field = getField("Destination Sec Postal Code");
			showField(field);

			field = getField("Destination Sec ZIP Code Extension");
			showField(field);

			field = getField("Destination Sec Geo Code");
			showField(field);

			System.out.println("");
			System.out.println(" An amount (gross or tax)");
			field = getField("Line Item Amount");
			showField(field);
		}
	}

	/**
	 * Retrieve the fields from the buffer.
	 * @param buffer the buffer with the fields information.
	 */
	private void getFields(StringBuffer buffer) {

		for (int i = 0; i < fields.size(); i++) {

			Field field = (Field) fields.get(i);

			if (LOG_GET_FIELDS) {
				Out.print("Copying [" + field.getPosition() + "-"
						+ (field.getPosition() + field.getLength()) + "] = [");
			}
			String value = buffer.substring(field.getPosition(),
					field.getPosition() + field.getLength());
			if (LOG_GET_FIELDS) {
				Out.println(value + "]");
			}

			field.setValue(value);
		}
	}

	/**
	 * Show the fields values.
	 */
	private void showFields() {

		for (int i = 0; i < fields.size(); i++) {

			Field field = (Field) fields.get(i);

			showField(field);
		}
	}

	/**
	 * Show a single field.
	 * @param field the field bean.
	 */
	private void showField(Field field) {

		Out.print(field.getField() + "[" + field.getValue() + "]");

		if (!field.getValue().trim().equals("")) {
			Out.println("         <---------------------------------------");
		} else {
			Out.println("");
		}
	}

	/**
	 * Get the value of a field.
	 * @param fieldName The field name.
	 * @return The value of the field.
	 */
	private String getFieldValue(String fieldName) {

		for (int i = 0; i < fields.size(); i++) {

			Field field = (Field) fields.get(i);

			if (field.getField().equals(fieldName)) {

				return field.getValue();
			}

		}

		return null;
	}

	/**
	 * Get a specific field.
	 * @param fieldName The field name.
	 * @return The field bean.
	 */
	private Field getField(String fieldName) {

		for (int i = 0; i < fields.size(); i++) {

			Field field = (Field) fields.get(i);

			if (field.getField().equals(fieldName)) {

				return field;
			}
		}

		return null;
	}

	/**
	 * Guarantee the correctness of fields
	 */
	private void validate() {

		int position = 0;

		for (int i = 0; i < fields.size(); i++) {

			Field field = (Field) fields.get(i);

			if (position != field.getPosition()) {

				Err.println("Error on " + field.getField());

				System.exit(1);

			} else {

				position += field.getLength();
			}
		}
	}

	/**
	 * Write the fields to a configuration file.
	 * @param fileName The file name to write.
	 */
	private void writeParameterFile(String fileName) {

		StringBuffer content = new StringBuffer();

		for (int i = 0; i < fields.size(); i++) {

			Field field = (Field) fields.get(i);

			content.append(field.getField() + "[" + field.getValue() + "]"
					+ "<" + field.getPosition() + ">" + "(" + field.getLength()
					+ ")" + "\r\n");
		}

		FileSupport.writeTextFile(fileName, content.toString());
	}

	/**
	 * Read the parameters from a file.
	 * @param fileName the file name.
	 */
	private void readParameterFile(String fileName) {

		String[] content = FileSupport.readAllLines(fileName);

		ArrayList newFields = new ArrayList();

		if (LOG_READ_PARAMETER) {
			Out.println("Reading parameters from file: " + fileName);
		}

		for (int i = 0; i < content.length; i++) {

			int indexStartField = content[i].indexOf('[');
			int indexEndField = content[i].indexOf(']');

			int indexStartFieldPosition = content[i].indexOf('<');
			int indexEndFieldPosition = content[i].indexOf('>');

			int indexStartFieldSize = content[i].indexOf('(');
			int indexEndFieldSize = content[i].indexOf(')');

			if (LOG_READ_PARAMETER) {
				Out.println(content[i]);
			}

			String name = content[i].substring(0, indexStartField);
			if (LOG_READ_PARAMETER) {
				Out.println("FieldName[" + name + "]");
			}

			String value = content[i].substring(indexStartField + 1,
					indexEndField);
			if (LOG_READ_PARAMETER) {
				Out.println("FieldValue[" + value + "]");
			}

			String position = content[i].substring(indexStartFieldPosition + 1,
					indexEndFieldPosition);
			if (LOG_READ_PARAMETER) {
				Out.println("FieldPosition[" + position + "]");
			}

			String size = content[i].substring(indexStartFieldSize + 1,
					indexEndFieldSize);
			if (LOG_READ_PARAMETER) {
				Out.println("FieldSize[" + size + "]");
			}

			Field field = new Field(new String(name),
					Integer.parseInt(position), Integer.parseInt(size));
			field.setValue(new String(value));
			newFields.add(field);
		}

		fields = newFields;
	}

	/**
	 * Generate the concatenated string sequence.
	 * @param fileName The target file name.
	 */
	private void generateString(String fileName) {

		StringBuffer output = new StringBuffer();

		for (int i = 0; i < fields.size(); i++) {

			Field field = (Field) fields.get(i);

			if (LOG_GENERATE_STRINGS) {
				Out.println("Inserting " + field.getField() + "["
						+ field.getValue() + "] at " + field.getPosition()
						+ " with length " + field.getLength());
			}
			output.append(field.getValue());

		}

		FileSupport.writeTextFile(fileName, output.toString());

	}

	private void fullTest() {

		StringBuffer buffer = null;

		validate();

		// loadVerazipInputFormat();
		// loadVerazipOutputormat();
		loadTaxwareInputFormat();
		// loadTaxwareOutputFormat();

		buffer = FileSupport.readTextFile(DIRECTORY + fileName);

		getFields(buffer);

		showFields();

		writeParameterFile("c:/parameters.txt");

		readParameterFile("c:/parameters.txt");

		generateString("c:/targetString.txt");

		showRequiredFields();
	}

	/**
	 * Execute the string concatenation class.
	 * @param args No arguments
	 */
	public static void main(String[] args) {

		StringConcatenation input = new StringConcatenation();

		input.fullTest();
	}
}
