/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.textbrowser;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;

/**
 * Text browser
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */

public class TextBrowser {
	private String windowTitleFileName = "No File";

	private List textFiles = new ArrayList();
	private int currentFile = -1;
	private StringBuffer content = new StringBuffer();
	private JTextPane textPane = new JTextPane();
	private JScrollPane scrollPane;
	private JFrame mainFrame;
	private JButton nextButton;
	private JButton previousButton;
	private JComboBox comboBox;
	private JLabel statusLabel = new JLabel("");

	private void ConstructWindow() {

		getTextFiles();

		mainFrame = new JFrame(windowTitleFileName);
		mainFrame.setSize(700, 700);
		textPane.setSize(800, 900);
		JPanel mainPanel = new JPanel();

		mainFrame.setContentPane(mainPanel);

		mainPanel.setLayout(new BorderLayout());
		scrollPane = new JScrollPane(textPane);
		mainPanel.add(scrollPane, BorderLayout.CENTER);
		nextButton = new JButton(">");
		mainPanel.add(nextButton, BorderLayout.EAST);
		previousButton = new JButton("<");
		mainPanel.add(previousButton, BorderLayout.WEST);
		textPane.setEditable(false);
		mainPanel.add(statusLabel, BorderLayout.SOUTH);

		comboBox = new JComboBox(textFiles.toArray());

		mainPanel.add(comboBox, BorderLayout.NORTH);

		class ComboListener implements ActionListener {
			public void actionPerformed(ActionEvent e) {
				statusLabel.setText("Reading File. Please Wait...");

				String absoluteFilename = ((JComboBox) e.getSource())
						.getSelectedItem().toString();

				try {

					FileReader fileReader = new FileReader(new File(
							absoluteFilename));
					BufferedReader bufferedReader = new BufferedReader(
							fileReader);

					// Content of the file
					content = new StringBuffer();
					// Current line
					String line = new String();
					while ((line = bufferedReader.readLine()) != null) {

						content.append(line);
						content.append("\r\n");
						// if (lines > 50)
						// break;
					}
				} catch (FileNotFoundException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				mainFrame.setTitle(absoluteFilename);

				// System.out.println(content.toString());
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						textPane.setText(content.toString());
						textPane.select(0, 0);
					}
				});

				for (int i = 0; i < textFiles.size(); i++) {

					if (((JComboBox) e.getSource()).getSelectedItem()
							.toString().equals(
									((File) textFiles.get(i)).toString()))
						currentFile = i;
				}

				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						// try {
						// Thread.sleep(100);
						// } catch (InterruptedException e) {
						// // TODO Auto-generated catch block
						// e.printStackTrace();
						// }
						statusLabel.setText("");
					}
				});

			}
		}
		;

		comboBox.addActionListener(new ComboListener());

		class ButtonListener implements ActionListener {
			public void actionPerformed(ActionEvent e) {

				if (textFiles.size() == 0)
					return;
				if (e.getSource() == nextButton) {

					currentFile++;
					if (currentFile == textFiles.size())
						currentFile = 0;
				} else {
					currentFile--;
					if (currentFile < 0) {
						currentFile = textFiles.size() - 1;
					}
				}
				statusLabel.setText("Reading File. Please Wait...");

				int lines = 0;

				try {
					FileReader fileReader = new FileReader((File) textFiles
							.get(currentFile));
					BufferedReader bufferedReader = new BufferedReader(
							fileReader);

					// Content of the file
					content = new StringBuffer();
					// Current line
					String line = new String();
					while ((line = bufferedReader.readLine()) != null) {
						lines++;
						content.append(line);
						content.append("\r\n");
						// if (lines > 50)
						// break;
					}
				} catch (FileNotFoundException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}

				mainFrame.setTitle(((File) textFiles.get(currentFile))
						.toString());

				comboBox.setSelectedIndex(currentFile);

				// System.out.println(content.toString());
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						textPane.setText(content.toString());
						textPane.select(0, 0);
					}
				});

				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						// try {
						// Thread.sleep(100);
						// } catch (InterruptedException e) {
						// // TODO Auto-generated catch block
						// e.printStackTrace();
						// }
						statusLabel.setText("");
					}
				});

			}
		}
		nextButton.addActionListener(new ButtonListener());
		previousButton.addActionListener(new ButtonListener());

		mainFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(1);
			}
		});

		// mainFrame.pack();
		mainFrame.setVisible(true);

	}

	public void getTextFiles() {

		File localDirectorie = new File(System.getProperty("user.dir"));
		File[] files = localDirectorie.listFiles();

		for (int i = 0; i < files.length; i++) {
			if (files[i].isFile()) {
				String fileName = files[i].getName();

				if (fileName.indexOf(".") != 0) {
					StringTokenizer tokenizer = new StringTokenizer(fileName,
							".");
					tokenizer.nextElement();
					String extention = tokenizer.nextToken();
					// System.out.println(extention);
					if (extention.toLowerCase().equals("txt")) {
						// System.out.println("Adding " + files[i]);
						textFiles.add(files[i]);
						// System.out.println("Actual size is " +
						// textFiles.size());
					}

				}

			}
		}
	}

	public static void main(String[] args) {
		TextBrowser browser = new TextBrowser();
		browser.ConstructWindow();

	}
}
