/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Calendar;

/**
 * Manage a simple output to file.
 *  
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class ExecutionStream {

    private Calendar calendar = Calendar.getInstance();

    private String outputDirectory = UtilityConfiguration.WRITE_OUT_MESSAGES_TO_ROOT ? "/" : "";
    private boolean outputDirectorySet = false;

    private String month = (calendar.get(Calendar.MONTH) + 1) < 10 ? "0"
        + (calendar.get(Calendar.MONTH) + 1) : ""
        + (calendar.get(Calendar.MONTH) + 1);
    private String day = (calendar.get(Calendar.DAY_OF_MONTH) < 10) ? "0"
        + calendar.get(Calendar.DAY_OF_MONTH) : ""
        + calendar.get(Calendar.DAY_OF_MONTH);
    private String hour = calendar.get(Calendar.HOUR_OF_DAY) < 10 ? "0"
        + calendar.get(Calendar.HOUR_OF_DAY) : ""
        + calendar.get(Calendar.HOUR_OF_DAY);
    private String minute = calendar.get(Calendar.MINUTE) < 10 ? "0"
        + calendar.get(Calendar.MINUTE) : "" + calendar.get(Calendar.MINUTE);
    private String second = calendar.get(Calendar.SECOND) < 10 ? "0"
        + calendar.get(Calendar.SECOND) : "" + calendar.get(Calendar.SECOND);

    private String outputFileNameSufix = "";
    private String fileDate = "" + calendar.get(Calendar.YEAR) + month + day
        + "_" + hour + minute + second;
    private boolean outputFileNameSufixSet = false;

    private String outputFileNamePrefix = InternalResource
            .get(InternalResource.EXECUTION);

    private File outputFile = null;

    /**
     * Inform the current file name
     * @return the absolute file name
     */
    protected String getFileName() {

        return outputDirectory + outputFileNamePrefix + "_"
            + outputFileNameSufix + "_" + fileDate + ".txt";
    }

    /**
     * Write the message to a file.
     * @param message the message
     * @param appendNewLine true to append a new line and false otherwise
     */
    protected synchronized void writeMessage(String message, boolean appendNewLine) {

        if (outputFile == null) {
            outputFile = new File(getFileName());

            if (!outputFile.exists()) {
                try {
                    outputFile.createNewFile();
                }
                catch(IOException e) {
                    // Silence
                    return;
                }
            }
        }
        FileOutputStream fileoutputstream;
        try {
            fileoutputstream = new FileOutputStream(outputFile, true);
        }
        catch(FileNotFoundException e) {
            // Silence
            return;
        }

        byte outputBytes[] = message.getBytes();

        byte newLineBytes[] = CommonConstants.NEW_LINE.getBytes();

        try {

            fileoutputstream.write(outputBytes);

            if (appendNewLine) {
                fileoutputstream.write(newLineBytes);
            }

            fileoutputstream.flush();

            fileoutputstream.close();
        }
        catch(IOException e) {
            // Silence
            return;
        }

    }


    /**
     * Print a message
     * @param message the message
     */
    protected void print(String message) {

        writeMessage(message, false);
    }

    /**
     * Print a message
     * @param message the message
     */
    protected void print(StringBuffer message) {

        writeMessage(message.toString(), false);
    }

    /**
     * Print a message followed by a new line
     * @param message the message
     */
    protected void println(String message) {

        writeMessage(message, true);
    }

    /**
     * Print a message followed by a new line
     * @param message the message
     */
    protected void println(StringBuffer message) {

        writeMessage(message.toString(), true);
    }

    /**
     * Indicate the directory to create the output file. 
     * The default is C:/
     * @param outputDirectory The target directory
     */
    protected synchronized void setOutputDirectory(String outputDirectory) {

        if (!outputDirectorySet) {
            outputDirectorySet = true;
            this.outputDirectory = outputDirectory;
        }
    }

    /**
     * Indicate the output file suffix.
     * @param outputFileNameSufix The target file suffix.
     */
    protected synchronized void setOutputFileNameSufix(String outputFileNameSufix) {

        if (!outputFileNameSufixSet) {
            outputFileNameSufixSet = true;
            this.outputFileNameSufix = outputFileNameSufix;
        }
    }
}
