/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */
package com.carlosbcruz.util;

/**
 * Allows the navigation on a string line by line.
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class LineTokenizer {

	private StringBuffer content = new StringBuffer();
	private int index = 0;
	private boolean foundNewLine = false;

	/**
	 * Default constructor.
	 * @param content The text to navigate line by line.
	 */
	public LineTokenizer(StringBuffer content) {
		super();
		this.content = content;
	}

	/**
	 * Indicate if there are more lines to read
	 * @return true if there are more lines to read.
	 */
	public boolean hasMoreTokens() {

		return index < content.length();
	}

	/**
	 * @return the foundNewLine
	 */
	public boolean isFoundNewLine() {
		return foundNewLine;
	}

	/**
	 * Provide the next line. If the line is empty than returns an empty String.
	 * @return The next line.
	 * @throws LineTokenizerException If there is a problem with the file format.
	 */
	public String nextToken() throws LineTokenizerException {

		int currentIndex = index;

		while (index < content.length()) {

			// If this is the end of the line
			if (content.charAt(index) == '\r' || content.charAt(index) == '\n') {

				foundNewLine = true;

				// If it is an empty line.
				if (currentIndex == index) {

					if (content.charAt(index) == '\r') {

						// Do a sanity check on file format. Validate DOS format.
						if (index + 1 >= content.length()
								|| content.charAt(index + 1) != '\n') {
							throw new LineTokenizerException(
									InternalResource
											.get(InternalResource.INCORRECT_FILE_FORMAT));
						}

						index += 2;
					} else {

						// Go to the next character.
						index++;
					}
					// Return an empty line.
					return new String();

				} else {

					String line = content.substring(currentIndex, index);

					if (content.charAt(index) == '\r') {

						// Do a sanity check on file format. Validate DOS format.
						if (index + 1 >= content.length()
								|| content.charAt(index + 1) != '\n') {
							throw new LineTokenizerException(
									InternalResource
											.get(InternalResource.INCORRECT_FILE_FORMAT));
						}

						index += 2;
					} else {

						// Go to the next character.
						index++;
					}

					// Return the line without the new line.
					return line;
				}
			} else {
				foundNewLine = false;
			}

			index++;
		}

		// Return the line until the end of the text.
		return content.substring(currentIndex, index);
	}
}
