/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */
package com.carlosbcruz.util;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;

/**
 * Simple Swing Utilities.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class SwingUtil {

	/**
	 * Indicate the center of the screen
	 * 
	 * @return the center point.
	 */
	public static Point getCenterOfScreen() {

		// Create a temporary frame in the center of the screen.
		Toolkit toolKit = Toolkit.getDefaultToolkit();
		Dimension screenSize = toolKit.getScreenSize();
		int middleX = screenSize.width / 2;
		int middleY = screenSize.height / 2;
		return new Point(middleX, middleY);
	}

	/**
	 * Inform the screen width.
	 * 
	 * @return The screen width.
	 */
	public static int getScreenWidth() {

		Toolkit toolKit = Toolkit.getDefaultToolkit();
		Dimension screenSize = toolKit.getScreenSize();
		return screenSize.width;
	}

	/**
	 * Inform the screen height.
	 * 
	 * @return The screen height.
	 */
	public static int getScreenHeight() {

		Toolkit toolKit = Toolkit.getDefaultToolkit();
		Dimension screenSize = toolKit.getScreenSize();
		return screenSize.height;
	}

	/**
	 * Retrieve an empty frame centered in the screen.
	 * 
	 * @return the frame.
	 */
	public static JFrame retrieveCenteredTemporaryFrame() {

		JFrame tempFrame = new JFrame();
		tempFrame.setLocation(getCenterOfScreen().x, getCenterOfScreen().y);
		return tempFrame;
	}

	/**
	 * Load an image.
	 * 
	 * @param imageLocation
	 *            The image location.
	 * @return The image.
	 */
	public static ImageIcon loadImage(String imageLocation) {

		if (imageLocation == null)
			return null;

		// Retrieve the icon
		URL iconURL = SwingUtil.class.getResource(imageLocation);
		if (iconURL != null) {
			return new ImageIcon(iconURL);
		}

		ExceptionSupport.handleException(InternalResource.get(
				InternalResource.IMAGE_FILE_NOT_FOUND, imageLocation));

		return null;
	}

	/**
	 * Link a label and a check box. If the label is clicked than
	 * the check box value changes.
	 *  
	 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
	 */
	class LabelCheckBoxLink extends MouseAdapter {

		JCheckBox checkBox;

		/**
		 * Default constructor.
		 * @param checkBox The check box to link with the label.
		 */
		LabelCheckBoxLink(JCheckBox checkBox) {
			this.checkBox = checkBox;
		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent)
		 */
		public void mouseClicked(MouseEvent e) {
			
			checkBox.setSelected(!checkBox.isSelected());
		}
	}

	/**
	 * Link a label and a check box. If the label is clicked than
	 * the check box value changes.
	 * @param label The label to be be linked with the check box.
	 * @param checkBox The check box to be linked with the label.
	 */
	public static void link(JLabel label, JCheckBox checkBox) {

		label.addMouseListener(new SwingUtil().new LabelCheckBoxLink(checkBox));
	}
}
